# -*- encoding:utf-8 -*-
import time, os
from datetime import datetime
from hashlib import md5
#import xlsxwriter
#from apps.resource.models import ResourceImage

#from main.settings import STATICFILES_DIRS


class Jt:
    # 判断是否是数字，包括浮点型
    @staticmethod
    def is_number(s):
        try:
            float(s)
            return True
        except ValueError:
            pass

        try:
            import unicodedata
            unicodedata.numeric(s)
            return True
        except (TypeError, ValueError):
            pass

        return False

    @staticmethod
    # 生成交易号：2位数（当前年份后2位数字）+8位数（当前时间戳去头2位）+6位数（用户名 经过hash crc16生成的 4位十六进制 转成5位数 然后头为补0）
    def make_unicode_16(salt=''):
        # 当前时间戳
        date_time = time.localtime(time.time())
        # 截取第3位到第4位
        year_code = str(date_time.tm_year)[2:4]

        # 当前时间戳
        timestamp = str(int(time.time()))
        # 截取第3位到第10位
        timestamp_code = timestamp[2:10]

        # 十六进制校验码
        crc_hex = Jt.crc16(salt) if salt else '0'
        # 十六进制转十进制
        crc_int = int(crc_hex, 16)
        # 头位补0
        crc_code = str('000000' + str(crc_int))[-6:]
        unicode = year_code + timestamp_code + crc_code

        return unicode

    # 生成日期yyyymmdd（8位）+时间（6位）hhmmss+毫秒（3位）=17位日期时间型数字
    @staticmethod
    def make_datetime_17():
        # 8位数+6+3=17位数
        t = time
        ms = int((t.time() - int(t.time()))*1000)
        return t.strftime('%Y%m%d%H%M%S',) + str(ms)

    @staticmethod
    def make_datetime_14():
        return time.strftime('%Y%m%d%H%M%S', )

    # crc16
    @staticmethod
    def crc16(x):
        a = 0xFFFF
        b = 0xA001
        for byte in x:
            a ^= ord(byte)
            for i in range(8):
                last = a % 2
                a >>= 1
                if last == 1:
                    a ^= b
        s = hex(a).upper()

        return s[2:6]

    # 文件生成md5
    @staticmethod
    def make_file_md5(file):
        if not os.path.exists(file):
            return None
        f = open(file, "rb")
        m = md5()
        m.update(f.read())
        f.close()
        result = m.hexdigest()
        return result

    # # 将数据导出为Excel
    # @staticmethod
    # def write_data_to_excel(save_dir, filename, format, header_list, data_list):
    #     '''
    #     瑛式备注
    #     :param save_dir: 保存文件的目录
    #     :param filename: 生成文件的文件名
    #     :param format: 生成文件的文件格式，只能是xlsx,xls,csv
    #     :param header_list: 文件的表头，即第一行标题
    #     :return:
    #     '''
    #
    #     folder_name = os.path.join(STATICFILES_DIRS[0], save_dir)
    #     # 没有该目录，则创建
    #     Jt.make_dir(folder_name=folder_name)
    #
    #     save_path = os.path.join(STATICFILES_DIRS[0], save_dir + filename + '.' + format)
    #
    #     # 打开文件
    #     wb = xlsxwriter.Workbook(save_path)
    #
    #     # 添加名字为Sheet1的Sheet
    #     ws = wb.add_worksheet('Sheet1')
    #
    #     # 设置表头
    #     row_num = 0
    #     columns = header_list
    #     for col_num in range(len(columns)):
    #         # 表头写入第一行
    #         ws.write(row_num, col_num, columns[col_num])
    #
    #     for index, it in enumerate(data_list):
    #         ws.write(index + 1, 0, it[0])  # A2 写入第A列数据
    #         ws.write(index + 1, 1, it[1])  # B2 写入第B列数据
    #         ws.write(index + 1, 2, it[2])  # C2 写入第C列数据
    #         ws.write(index + 1, 3, it[3])  # D2 写入第D列数据
    #         ws.write(index + 1, 4, it[4])  # E2 写入第E列数据
    #         ws.write(index + 1, 5, it[5])  # F2 写入第F列数据
    #         ws.write(index + 1, 6, it[6])  # G2 写入第G列数据
    #         ws.write(index + 1, 7, it[7])  # H2 写入第H列数据
    #         ws.write(index + 1, 8, it[8])  # I2 写入第I列数据
    #     wb.close()
    #
    #     return 1

    # 如果没有目录，则创建目录
    @staticmethod
    def make_dir(folder_name):
        if not os.path.exists(folder_name):
            os.makedirs(folder_name)
        return 1










from django.contrib import admin
# 引入用户平台
from .models import *


# #
#
class InvoiceAdmin(admin.ModelAdmin):
    fields = (
        'id', 'user_id', 'thread_id', 'invoice_time', 'invoice_type', 'invoice_number', 'invoice_price', 'tax_rate',
        'invoice_tax',
        'invoice_untaxed', 'tax_number', 'operator_user_id', 'remark', 'invoice_status', 'destroy_name', 'destroy_date',
        'destroy_reason', 'destroy_operator_id', 'receive_email', 'receive_phone', 'invoice_snapshot')
    list_display = (
        'id', 'user_id', 'thread_id', 'invoice_time', 'invoice_type', 'invoice_number', 'invoice_price', 'tax_rate',
        'invoice_tax',
        'invoice_untaxed', 'tax_number', 'operator_user_id', 'remark', 'invoice_status', 'destroy_name', 'destroy_date',
        'destroy_reason', 'destroy_operator_id', 'receive_email', 'receive_phone', 'invoice_snapshot')
    search_fields = (
        'user_id', 'invoice_number', 'invoice_type', 'receive_email')
    readonly_fields = ['id']


# #
#
class EextendFieldAdmin(admin.ModelAdmin):
    fields = ('id', 'field_index', 'field', 'value', 'type', 'unit', 'config', 'description', 'default')
    list_display = (
        'id', 'field_index', 'field', 'value', 'type', 'unit', 'config', 'description', 'default')
    search_fields = (
        'id', 'field_index', 'field', 'value', 'type', 'unit', 'config', 'description', 'default')
    readonly_fields = ['id']


#
# #
admin.site.register(Invoice, InvoiceAdmin)
admin.site.register(InvoiceExtendField, EextendFieldAdmin)



# _*_coding:utf-8_*_

import os, logging, time, json, copy

from django.views.decorators.http import require_http_methods
from rest_framework.decorators import api_view
from rest_framework.views import APIView

from xj_role.services.user_group_service import UserGroupService
from xj_thread.services.thread_list_service import ThreadListService
from xj_user.services.user_detail_info_service import DetailInfoService
from xj_user.services.user_service import UserService
from ..utils.custom_tool import request_params_wrapper, flow_service_wrapper, format_params_handle
from ..utils.user_wrapper import user_authentication_wrapper
from ..utils.model_handle import util_response
from ..services.invoice_service import InvoiceService
from ..utils.utility_method import extract_values

logger = logging.getLogger(__name__)


class InvoiceApi(APIView):  # 或继承(APIView)

    # 发票添加
    @api_view(['POST'])
    @user_authentication_wrapper
    @request_params_wrapper
    def add(self, *args, user_info, request_params, **kwargs, ):
        params = request_params
        # ============   字段验证处理 start ============
        user_id = user_info.get("user_id")
        params.setdefault("user_id", user_id)  # 用户ID
        invoice_set, err = InvoiceService.add(params)
        if err is None:
            return util_response(data=invoice_set)
        return util_response(err=47767, msg=err)

    # 发票批量添加
    @api_view(['POST'])
    @user_authentication_wrapper
    @request_params_wrapper
    def batch_add(self, *args, user_info, request_params, **kwargs, ):
        params = request_params
        # ============   字段验证处理 start ============
        invoice_set, err = InvoiceService.batch_add(params)
        if err is None:
            return util_response(data=invoice_set)
        return util_response(err=47767, msg=err)

    # 发票修改
    @api_view(['PUT'])
    @user_authentication_wrapper
    @request_params_wrapper
    def edit(self, *args, user_info, request_params, **kwargs, ):
        params = request_params
        user_id = user_info.get("user_id")
        platform_id = user_info.get("platform_id")
        invoice_id = params.get("invoice_id", 0)
        invoice_set, err = InvoiceService.edit(params, invoice_id)
        if err is None:
            return util_response(data=invoice_set)
        return util_response(err=47767, msg=err)

    # 发票列表
    @api_view(['GET'])
    @user_authentication_wrapper
    @request_params_wrapper
    def list(self, *args, user_info, request_params, **kwargs, ):
        if request_params is None:
            request_params = {}
        if user_info:
            request_params.setdefault("user_id", user_info.get("user_id"))
        else:
            return util_response(err=6001, msg="非法请求，请您登录")

        # ================== 信息id列表反查询发票 start===============================
        thread_params = format_params_handle(
            param_dict=request_params,
            filter_filed_list=["title", "subtitle", "access_level", "author", "customer_code"],
            is_remove_empty=True
        )
        if thread_params:
            thread_ids, err = ThreadListService.list(params=thread_params)
            if not err:
                request_params["thread_id_list"] = extract_values(thread_ids['list'], "id")

            if isinstance(request_params.get("thread_id_list"), list) and len(request_params["thread_id_list"]) == 0:
                request_params["thread_id_list"] = [0]
        # ================== 信息id列表反查询发票 end ===============================

        # ================== 分组id列表反查询发票 start ===============================
        group_id = request_params.get("group_id", "")
        if group_id:
            group, err = UserGroupService.get_user_ids_by_group(group_id)
            if not err:
                request_params["user_id_list"] = group
        # ================== 分组id列表反查询发票 end ===============================

        # ================== 用户id列表反查询发票 start===============================
        user_params = format_params_handle(
            param_dict=request_params,
            filter_filed_list=["user_name"],
            is_remove_empty=True
        )
        if user_params:
            user_ids, err = DetailInfoService.get_list_detail(params=user_params)
            if not err:
                request_params["user_id_list"] = extract_values(user_ids['list'], 'user_id')

            if isinstance(request_params.get("user_id_list"), list) and len(
                    request_params["user_id_list"]) == 0:
                request_params["user_id_list"] = [0]
        # ================== 用户id列表反查询发票 end ===============================

        invoice_set, err = InvoiceService.list(request_params)

        if err is None:
            return util_response(data=invoice_set)

        return util_response(err=47767, msg=err)

    # 发票详细
    @api_view(['GET'])
    @user_authentication_wrapper
    @request_params_wrapper
    def detail(self, *args, user_info, request_params, **kwargs, ):
        params = request_params
        # ============   字段验证处理 start ============
        user_id = user_info.get("user_id")
        params.setdefault("user_id", user_id)  # 用户ID
        invoice_set, err = InvoiceService.detail(params)
        if err is None:
            return util_response(data=invoice_set)
        return util_response(err=47767, msg=err)

    # 发票审批
    @api_view(['POST'])
    @user_authentication_wrapper
    @request_params_wrapper
    @flow_service_wrapper
    def examine_approve(self, *args, user_info, request_params, **kwargs, ):
        params = request_params
        # ============   字段验证处理 start ============
        user_id = user_info.get("user_id")
        params.setdefault("user_id", user_id)  # 用户ID
        invoice_set, err = InvoiceService.examine_approve(params)
        if err is None:
            return util_response(data=invoice_set)
        return util_response(err=47767, msg=err)



from django.shortcuts import render
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import serializers
from .models import *
from rest_framework.permissions import IsAuthenticated, AllowAny
from rest_framework.authentication import TokenAuthentication
from django.db.models import Q
from django.db.models import F
import json

from django.http import JsonResponse
# Create your views here.


class FinanceAPISerializer(serializers.ModelSerializer):   # 继承自ModelSerializer类
    """ 序列化数据的类，根据model表来获取字段 """
    class Meta:
        model = Transact
        fields = '__all__'


class FinanceListAPIView(APIView):
    """ REST framework的APIView实现获取card列表 """
    # authentication_classes = (TokenAuthentication,)  # token认证
    # permission_classes = (IsAuthenticated,)   # IsAuthenticated 仅通过认证的用户
    permission_classes = (AllowAny,)  # 允许所有用户
    params = None

    def get(self, request, format=None):
        self.params = request.query_params  # 返回QueryDict类型

        if 'uid' not in self.params or self.params['uid'].isspace():
            return Response({'err': 0, 'msg': '缺少uid', 'data': [], 'request': self.params, })

        page = int(self.params['page']) - 1 if 'page' in self.params else 0
        size = int(self.params['size']) if 'size' in self.params else 10

        """ Return a list of all users. """
        transacts = Transact.objects.all().filter(Q(account=self.params['uid']) | Q(their_account=self.params['uid'])).order_by('transact_time')
        total = transacts.count()
        now_pages = transacts[page*size:page*size+size] if page >= 0 else transacts
        data = now_pages.annotate(
            self_name=F('account__full_name'),
            their_name=F('their_account__full_name'),
            time=F('transact_time'),
            platform_name=F('platform__platform_name'),
            currency_name=F('currency__currency'),
            payment=F('pay_mode__pay_mode'),
            order_id=F('platform_order_id'),
        ).values(
            'id',
            'transact_id',
            'self_name',
            'their_name',
            'time',
            'platform_name',
            'order_id',
            'opposite_account',
            'summary',
            'currency_name',
            'income',
            'outgo',
            'balance',
            'payment',
        )

        serializer = FinanceAPISerializer(transacts, many=True)
        return Response({
            'err': 0,
            'msg': 'OK',
            'data': {'total': total, 'list': data, },
            'request': self.params,
            # 'serializer': serializer.data,
        })

        # m_transacts = Transact.objects.values()
        # m_result_transacts = {}
        # m_result_transacts['list'] = list(m_transacts)
        #
        # m_BaseInfo = BaseInfo.objects.values()
        # m_result = {}
        # m_result['list'] = list(m_BaseInfo)

        # return Response({'err': 0, 'msg': 'OK', 'data': serializer.data, 'm_result': m_result, 'm_result_transacts': m_result_transacts})

        # transacts = Transact.objects.all().values()
        # data = {}
        # data['list'] = list(transacts)
        #
        # # return JsonResponse(data)

    def post(self, request):
        self.params = request.query_params

        return Response({
            'err': 0,
            'msg': 'OK',
            'data': {},
            'request': self.params,
        })



# _*_coding:utf-8_*_

import os, logging, time, json, copy

from django.views.decorators.http import require_http_methods
from rest_framework.decorators import api_view
from rest_framework.views import APIView

from ..utils.custom_tool import request_params_wrapper, flow_service_wrapper
from ..utils.user_wrapper import user_authentication_wrapper
from ..utils.model_handle import util_response
from ..services.invoice_service import InvoiceService

logger = logging.getLogger(__name__)


class InvoiceExternalApi(APIView):  # 或继承(APIView)

    # 发票添加
    @api_view(['POST'])
    @user_authentication_wrapper
    @request_params_wrapper
    def add(self, *args, user_info, request_params, **kwargs, ):
        params = request_params
        user_id = user_info.get("user_id")
        platform_id = user_info.get("platform_id")
        params.setdefault("user_id", user_id)  # 用户ID
        invoice_set, err = InvoiceService.add(params)
        if err is None:
            return util_response(data=invoice_set)
        return util_response(err=47767, msg=err)

    # 发票修改
    @api_view(['PUT'])
    @user_authentication_wrapper
    @request_params_wrapper
    def edit(self, *args, user_info, request_params, **kwargs, ):
        params = request_params
        user_id = user_info.get("user_id")
        platform_id = user_info.get("platform_id")
        invoice_id = params.get("invoice_id", 0)
        invoice_set, err = InvoiceService.edit(params, invoice_id)
        if err is None:
            return util_response(data=invoice_set)
        return util_response(err=47767, msg=err)

    # 发票列表
    @api_view(['GET'])
    @user_authentication_wrapper
    @request_params_wrapper
    def list(self, *args, user_info, request_params, **kwargs, ):
        params = request_params
        # ============   字段验证处理 start ============
        user_id = user_info.get("user_id")
        params.setdefault("user_id", user_id)  # 用户ID

        invoice_set, err = InvoiceService.list(params)

        if err is None:
            return util_response(data=invoice_set)

        return util_response(err=47767, msg=err)

    # 发票详细
    @api_view(['GET'])
    @user_authentication_wrapper
    @request_params_wrapper
    def detail(self, *args, user_info, request_params, **kwargs, ):
        params = request_params
        # ============   字段验证处理 start ============
        user_id = user_info.get("user_id")
        params.setdefault("user_id", user_id)  # 用户ID
        invoice_set, err = InvoiceService.detail(params.get("invoice_id", 0))
        if err is None:
            return util_response(data=invoice_set)
        return util_response(err=47767, msg=err)

    # 发票审批
    @api_view(['POST'])
    @user_authentication_wrapper
    @request_params_wrapper
    @flow_service_wrapper
    def examine_approve(self, *args, user_info, request_params, **kwargs, ):
        params = request_params
        # ============   字段验证处理 start ============
        user_id = user_info.get("user_id")
        params.setdefault("user_id", user_id)  # 用户ID
        invoice_set, err = InvoiceService.examine_approve(params)
        if err is None:
            return util_response(data=invoice_set)
        return util_response(err=47767, msg=err)




# _*_coding:utf-8_*_
from django.urls import re_path

from .apis.invoice_apis import InvoiceApi
from .apis.invoice_header_apis import InvoiceHeaderApi
from .apis.invoice_type_apis import InvoiceTypeApi
from .service_register import register

register()

urlpatterns = [
    re_path(r'^add/?$', InvoiceApi.add, name="发票添加"),  # 发票添加
    re_path(r'^batch_add/?$', InvoiceApi.batch_add, name="发票批量添加"),  # 发票批量添加
    re_path(r'^edit/?$', InvoiceApi.edit, name="发票编辑"),  # 编辑
    re_path(r'^list/?$', InvoiceApi.list, name="发票列表"),  # 列表
    re_path(r'^detail/?$', InvoiceApi.detail, name="发票详情"),  # 详情
    re_path(r'^examine_approve/?$', InvoiceApi.examine_approve, name="发票审批"),  # 发票审批

    re_path(r'^type_list/?$', InvoiceTypeApi.list, name="发票状态列表"),  # 列表

    re_path(r'^header_add/?$', InvoiceHeaderApi.add, name="发票抬头添加"),  # 发票抬头添加
    re_path(r'^header_detail/?$', InvoiceHeaderApi.detail, name="发票抬头详情"),  # 详情
    re_path(r'^header_edit/?$', InvoiceHeaderApi.edit, name="发票抬头编辑"),  # 编辑
    re_path(r'^header_list/?$', InvoiceHeaderApi.list, name="发票抬头列表"),  # 列表
    re_path(r'^header_delete/?$', InvoiceHeaderApi.delete, name="发票抬头删除"),  # 删除
]



from decimal import Decimal

from django.db.models import F
from django.forms import model_to_dict
from xj_thread.services.thread_list_service import ThreadListService
from ..utils.custom_tool import format_params_handle, force_transform_type

from ..models import *
from ..utils.join_list import JoinList
from xj_migrate.utils.utility_method import replace_dict_key


# 自定义序列化函数
def decimal_default(obj):
    if isinstance(obj, Decimal):
        return str(obj)  # 将 Decimal 对象转换为字符串
    raise TypeError(f"Object of type {obj.__class__.__name__} is not JSON serializable")


class InvoiceExternalService:

    @staticmethod
    def add(params: dict = None, **kwargs):
        """
        发票-外经证添加
        :param params: 添加参数子字典
        :param kwargs:
        :return:
        """
        # 参数整合与空值验证
        params, is_void = force_transform_type(variable=params, var_type="dict", default={})
        kwargs, is_void = force_transform_type(variable=kwargs, var_type="dict", default={})
        params.update(kwargs)
        # 过滤主表修改字段
        try:
            main_form_data = format_params_handle(
                param_dict=params.copy(),
                is_remove_empty=True,
                filter_filed_list=[
                    "thread_id|int",
                    "external_number",
                    "create_date|date",
                    "external_status_id|int",
                    "invoice_price|float",
                    "external_price",
                    "tax_date|date",
                    "end_date|date",
                    "external_address",
                    "remark",
                    "return_date|date",
                    'destroy_name',
                    "destroy_date"
                ],
                alias_dict={},
                is_validate_type=True
            )
        except ValueError as e:
            # 模型字段验证
            return None, str(e)

        # 必填参数校验
        must_keys = ["thread_id"]
        for i in must_keys:
            if not params.get(i, None):
                return None, str(i) + " 必填"

        # IO操作
        try:
            invoice_external = InvoiceExternal.objects.create(**main_form_data)
        except Exception as e:
            return None, f'''{str(e)} in "{str(e.__traceback__.tb_frame.f_globals["__file__"])}" : Line {str(
                e.__traceback__.tb_lineno)}'''

        return {"id": invoice_external.id}, None

    @staticmethod
    def edit(params: dict = None, external_id=None, search_param: dict = None, **kwargs):
        """
        发票-外经证编辑
        :param params: 修改的参数
        :param external_id: 需要修改的外经证主键
        :param search_param: 搜索参数, 服务层根据信息等其他搜索字段检索到需要修改的数据
        :return: data, err
        """
        # 空值检查
        params, is_pass = force_transform_type(variable=params, var_type="dict", default={})
        external_id, is_pass = force_transform_type(variable=external_id, var_type="int")
        search_param, is_pass = force_transform_type(variable=search_param, var_type="dict", default={})
        if not external_id and not search_param:
            return None, "无法找到要修改数据，请检查参数"

        # 搜索字段过滤
        if search_param:
            search_param = format_params_handle(
                param_dict=search_param,
                filter_filed_list=[
                    "external_id|int", "external_id_list|list", "thread_id|int", "thread_id_list|list",
                ],
                alias_dict={"external_id_list": "id__in", "thread_id_list": "thread_id__in"}
            )
        # 修改内容检查处理
        try:
            main_form_data = format_params_handle(
                param_dict=params,
                is_validate_type=True,
                is_remove_empty=True,
                filter_filed_list=[
                    "thread_id|int",
                    "external_number",
                    "create_date|date",
                    "external_status_id|int",
                    "invoice_price|float",
                    "external_price",
                    "tax_date|date",
                    "end_date|date",
                    "external_address",
                    "remark",
                    "return_date|date",
                    'destroy_name',
                    "destroy_date"

                ],
            )
        except ValueError as e:
            return None, str(e)
        # 构建ORM，检查是否存在可修改项目
        external_obj = InvoiceExternal.objects
        if external_id:
            external_obj = external_obj.filter(id=external_id)
        elif search_param:
            external_obj = external_obj.filter(**search_param)

        update_total = external_obj.count()
        if update_total == 0:
            return None, "没有找到可修改项目"

        # IO 操作
        try:
            main_form_data.pop("external_id", None)
            external_obj.update(**main_form_data)
        except Exception as e:
            return None, "修改异常:" + str(e)
        return external_obj.first().to_json(), None

    @staticmethod
    def list(params):

        page = int(params['page']) - 1 if 'page' in params else 0
        size = int(params['size']) if 'size' in params else 10
        external = InvoiceExternal.objects
        # invoice = invoice.order_by('-invoice_time')
        external = external.order_by('-id')
        params = format_params_handle(
            param_dict=params,
            is_remove_empty=True,
            filter_filed_list=[
                "id|int", "id_list|list", "thread_id|int", "user_id|int", "invoice_type_code",
                "thread_id_list|list",
                "invoice_number|int",
                "invoice_time_start|date", "invoice_time_end|date", "invoice_status"
            ],
            split_list=["thread_id_list", "id_list"],
            alias_dict={
                "create_date_start": "create_date__gte", "create_date_end": "create_date__lte",
                "thread_id_list": "thread_id__in", "id_list": "id__in",
                "external_status_code": "external_status__external_status_code__iexact"
            },
        )
        external = external.annotate(external_status_code=F("external_status__external_status_code"),
                                     external_status_value=F("external_status__external_status_value"))
        external = external.filter(**params).values()
        total = external.count()
        #
        current_page_set = external[page * size: page * size + size] if page >= 0 and size > 0 else external
        res_list = []
        for i, it in enumerate(current_page_set):
            it['order'] = page * size + i + 1
            it['create_date'] = it['create_date'].strftime("%Y-%m-%d %H:%M:%S") if it['create_date'] else it[
                'create_date']
            it['tax_date'] = it['tax_date'].strftime("%Y-%m-%d %H:%M:%S") if it['tax_date'] else it[
                'tax_date']
            it['end_date'] = it['update_time'].strftime("%Y-%m-%d %H:%M:%S") if it['end_date'] else it[
                'end_date']
            it['return_date'] = it['return_date'].strftime("%Y-%m-%d %H:%M:%S") if it['return_date'] else it[
                'return_date']
            it['destroy_date'] = it['destroy_date'].strftime("%Y-%m-%d %H:%M:%S") if it['destroy_date'] else it[
                'destroy_date']
            res_list.append(it)

        data = res_list

        thread_id_list = [item.get("thread_id", None) for item in data]
        thread_list, err = ThreadListService.search(thread_id_list, filter_fields="title")
        if thread_list:
            data = JoinList(data, thread_list, "thread_id", "id").join()

        return {'size': int(size), 'page': int(page + 1), 'total': total, 'list': data, }, None

    @staticmethod
    def detail(external_id):
        if not external_id:
            return None, "外经证id不能为空"
        external = InvoiceExternal.objects.filter(id=external_id)
        external = external.extra(select={'create_date': 'DATE_FORMAT(create_date, "%%Y-%%m-%%d %%H:%%i:%%s")',
                                          'tax_date': 'DATE_FORMAT(tax_date, "%%Y-%%m-%%d %%H:%%i:%%s")',
                                          'end_date': 'DATE_FORMAT(end_date, "%%Y-%%m-%%d %%H:%%i:%%s")',
                                          'return_date': 'DATE_FORMAT(return_date, "%%Y-%%m-%%d %%H:%%i:%%s")',
                                          'destroy_date': 'DATE_FORMAT(destroy_date, "%%Y-%%m-%%d %%H:%%i:%%s")'})

        external = external.first()
        if not external:
            return None, "无法找到要查询的数据，请检查参数"
        data = model_to_dict(external)
        data = replace_dict_key(data, 'id', 'invoice_id')
        external_status = InvoiceStatus.objects.filter(id=data['external_status_id']).first()
        if external_status:
            data.update(model_to_dict(external_status))
            data = replace_dict_key(data, 'id', 'external_status_id')
        thread_id_list = [data.get("thread_id", None)]
        thread_list, err = ThreadListService.search(thread_id_list, filter_fields="title")
        if thread_list:
            data.update(thread_list[0])
        return data, None








# encoding: utf-8
"""
@project: djangoModel->extend_service
@author:高栋天
@Email: sky4834@163.com
@synopsis: 扩展服务
@created_time: 2022/7/29 15:14
"""
from django.db.models import F

from ..models import *
from ..utils.custom_tool import write_to_log, force_transform_type, filter_result_field, format_params_handle


# 扩展字段增删改查
class InvoiceExtendService:
    @staticmethod
    def create_or_update(params=None, invoice_id=None, invoice_type_id=None, **kwargs):
        """
        信息表扩展信息新增或者修改
        :param params: 扩展信息，必填
        :param invoice_id: 发票ID，必填
        :param invoice_type_id: 沙盒ID, 非必填
        :return: None，err
        """
        # 参数合并，强制类型转换
        kwargs, is_void = force_transform_type(variable=kwargs, var_type="dict", default={})
        params, is_void = force_transform_type(variable=params, var_type="dict", default={})
        params.update(kwargs)
        # 不存在信息ID 无法修改
        invoice_id = invoice_id or params.pop("invoice_id", None)
        invoice_id, is_void = force_transform_type(variable=invoice_id, var_type="int")
        if invoice_id is None:
            return None, "扩展字段修改错误,invoice_id不可以为空"
        # 检查信息ID 是否正确
        invoice_obj = Invoice.objects.filter(id=invoice_id).first()
        if not invoice_obj:
            return None, "没有找到该主表信息，无法添加扩展信息"
        # 扩展字段映射, 如没有配置对应类别的扩展字段，则无法添加扩展数据。
        extend_fields = InvoiceExtendField.objects.values("field_index",
                                                          "default",
                                                          "field")
        if not extend_fields:
            return None, "没有配置扩展字段，无法添加扩展信息"

        extend_model_fields = [i.name for i in InvoiceExtendData._meta.fields if
                               not i.name == "invoice_id"]  # 扩展信息表的字段列表
        # 扩展数据替换
        extend_field_map = {item["field"]: item["field_index"] for item in extend_fields if
                            item["field_index"] in extend_model_fields}  # 得到合理的配置
        transformed_extend_params = {extend_field_map[k]: v for k, v in params.items() if
                                     extend_field_map.get(k)}  # {"自定义扩展字段":"123"} ==>> {"filed_1":"123"}
        # 修改或添加数据
        try:
            extend_obj = InvoiceExtendData.objects.filter(invoice_id=invoice_id)
            if not extend_obj:
                # 新增的时候，启用扩展字段参数设置默认值。
                # 注意：防止后台管理员配置错误,出现数据表不存在的字段。所以需要进行一次字段排除
                default_field_map = {item["field_index"]: item["default"] for item in extend_fields if
                                     (item["default"] and item["field_index"] in extend_model_fields)}
                for field_index, default in default_field_map.items():
                    transformed_extend_params.setdefault(field_index, default)
                if not transformed_extend_params:
                    return None, "没有可添加的数据，请检查扩展字段的默认值配置"

                # 添加扩展信息
                transformed_extend_params.setdefault('invoice_id_id', invoice_id)
                InvoiceExtendData.objects.create(**transformed_extend_params)
                return None, None
            else:
                if not transformed_extend_params:
                    return None, "没有可修改的数据"

                extend_obj.update(**transformed_extend_params)
                return None, None
        except Exception as e:
            write_to_log(prefix="信息表扩展信息新增或者修改异常", err_obj=e)
            return None, "信息表扩展信息新增或者修改异常:" + str(e)

    @staticmethod
    def get_extend_info(invoice_id_list: list = None):
        """
        获取映射后的扩展数据
        :param invoice_id_list: 发票ID列表
        :return: extend_list, err
        """
        # 参数类型校验
        invoice_id_list, is_void = force_transform_type(variable=invoice_id_list, var_type="list")
        if not invoice_id_list:
            return [], None

        # 扩展字段映射, 如没有配置对应类别的扩展字段，则无法添加扩展数据。
        extend_fields = list(InvoiceExtendField.objects.values("field_index", "field"))
        if not extend_fields:
            return [], None
        extend_field_map = {}
        for item in extend_fields:
            extend_field_map.update({item["field_index"]: item["field"]})
        # 查询出扩展原始数据
        try:
            invoice_extend_list = list(InvoiceExtendData.objects.filter(invoice_id__in=invoice_id_list).values())
        except Exception as e:
            return [], "获取扩展数据异常"
        # 处理获取到结果，字段替换
        try:
            finish_list = []
            for i in invoice_extend_list:
                # 进行替换
                finish_list.append(format_params_handle(
                    param_dict=i,
                    alias_dict=extend_field_map,
                    is_remove_null=False
                ))
            # 剔除不需要的字段
            finish_list = filter_result_field(
                result_list=finish_list,
                remove_filed_list=[i.name for i in InvoiceExtendData._meta.fields if not i.name == "invoice_id"]
            )
            return finish_list, None

        except Exception as e:
            write_to_log(prefix="获取映射后的扩展数据,数据拼接异常", err_obj=e,
                         content="invoice_id_list:" + str(invoice_id_list))
            return [], None



import sys
import time
import os
import datetime

from django.forms.models import model_to_dict

from ..models import *
from django.db.models import Q, F


class InvoiceHeaderService:

    @staticmethod
    def list(params):

        user_id = params.pop("user_id", "")
        sort = params.pop("sort", "-id")
        sort = sort if sort and sort in ["-id", "-sort", "id", "sort"] else "-id"

        currencies = InvoiceHeader.objects.filter(user_id=user_id, is_delete=0).order_by(
            sort)

        return list(currencies.values('id', 'head_type', 'company_name', 'tax_number', 'address', 'phone_number',
                                      'account_opening_bank',
                                      'account', 'sort')), None

    @staticmethod
    def edit(params):
        invoice_header_id = params.get('invoice_header_id', '')
        tax_number = params.get('tax_number', '')
        head_type = params.get('head_type', '')
        sort = params.get('sort', '')
        user_id = params.get('user_id', '')
        invoice_header_set = InvoiceHeader.objects.filter(Q(tax_number=tax_number) & ~Q(id=invoice_header_id))
        if not head_type:
            return None, "抬头类型不允许为空"
        if head_type == "CORPORATIONS" and not tax_number:
            return None, "税号不允许为空"
        if invoice_header_set.first():
            return None, "税号已存在"
        if sort:
            InvoiceHeader.objects.filter(user_id=user_id).update(**{"sort": 0})
        try:
            params.pop("invoice_header_id")
            InvoiceHeader.objects.filter(id=invoice_header_id).update(**params)
            return None, None
        except Exception as e:
            return None, "参数配置错误：" + str(e)

    @staticmethod
    def detail(params):
        invoice_header_id = params.get('invoice_header_id', '')
        if not invoice_header_id:
            return None, "id不能为空"
        try:
            invoice_header_set = InvoiceHeader.objects.filter(id=invoice_header_id).first()
            if invoice_header_set:
                return model_to_dict(invoice_header_set), None
            else:
                return None, "数据不存在"
        except Exception as e:
            return None, "参数配置错误：" + str(e)

    @staticmethod
    def delete(params):
        invoice_header_id = params.get('invoice_header_id', '')
        try:
            InvoiceHeader.objects.filter(id=invoice_header_id).update(**{"is_delete": 1})
            return None, None
        except Exception as e:
            return None, "参数配置错误：" + str(e)

    @staticmethod
    def add(params):
        tax_number = params.get('tax_number', '')
        head_type = params.get('head_type', '')
        sort = params.get('sort', '')
        user_id = params.get('user_id', '')
        if not head_type:
            return None, "抬头类型不允许为空"
        if head_type == "CORPORATIONS" and not tax_number:
            return None, "税号不允许为空"
        if sort:
            InvoiceHeader.objects.filter(user_id=user_id).update(**{"sort": 0})
        if tax_number:
            invoice_header_set = InvoiceHeader.objects.filter(tax_number=tax_number).first()
            if invoice_header_set is not None:
                return None, "税号已存在"
        try:
            InvoiceHeader.objects.create(**params)
            return None, None
        except Exception as e:
            return None, "参数配置错误：" + str(e)



import sys
import time
import os
import datetime

from ..models import *


class InvoiceTypeService:

    @staticmethod
    def list():
        invoice_type = InvoiceType.objects.values("id", "invoice_type_code", "invoice_type_value", "description")
        return list(invoice_type), None



# -*- coding: UTF-8 -*-

import logging, re, urllib.parse
from django.conf import settings
from .middleware import GlobalRequestMiddleware
from django.contrib.sites.shortcuts import get_current_site

logger = logging.getLogger(__name__)

# 获取当前域名
def getSiteDomain(request = None):
	request = request or GlobalRequestMiddleware.getRequest()
	if request:
		header = request.META.get("HTTP_SITE_ROUTER") or request.headers.get("Site-Router")
		if header and header in settings.ALLOWED_HOSTS:
			return header
		
		query = request.GET.get("site-router")
		if query and query in settings.ALLOWED_HOSTS:
			return query
		
		site = get_current_site(request)
		if site:
			return site.domain
	return None
#end - getSiteDomain

# 当前是否为移动端访问
def isMobile(request = None):
	request = request or GlobalRequestMiddleware.getRequest()
	userAgent = request.META.get("HTTP_USER_AGENT")
	if userAgent and re.search(r"(mobile|android|iphone|ipad|phone|micromessenger|alipay)", userAgent, re.IGNORECASE):
		return True
	
	return False
#end - isMobile

# 当前是否为微信浏览器访问
def isWeChat(request = None):
	request = request or GlobalRequestMiddleware.getRequest()
	if request.META.get("HTTP_WECHAT") == "1" or request.headers.get("wechat") == "1" or request.GET.get("wechat") == "1":
		return True
	
	userAgent = request.META.get("HTTP_USER_AGENT")
	if userAgent and re.search(r"(micromessenger)", userAgent, re.IGNORECASE):
		return True
	
	return False
#end - isWeChat

# 当前是否为小程序访问
def isMiniProgram(request = None):
	request = request or GlobalRequestMiddleware.getRequest()
	if request.META.get("HTTP_MINIPROGRAM") == "1" or request.headers.get("miniprogram") == "1" or request.GET.get("miniprogram") == "1":
		return True
	
	userAgent = request.META.get("HTTP_USER_AGENT")
	if userAgent and re.search(r"(miniprogram)", userAgent, re.IGNORECASE):
		return True
	
	return False
#end - isMiniProgram

# 当前是否为支付宝浏览器访问
def isAlipay(request = None):
	request = request or GlobalRequestMiddleware.getRequest()
	if request.META.get("HTTP_ALIPAY") == "1" or request.headers.get("alipay") == "1" or request.GET.get("alipay") == "1":
		return True
	
	userAgent = request.META.get("HTTP_USER_AGENT")
	if userAgent and re.search(r"(alipay)", userAgent, re.IGNORECASE):
		return True
	
	return False
#end - isAlipay

# 获取接口白名单域名
def getBlackListSite():
	site = list(settings.DATABASE_MAPPING.keys())
	if not site:
		return getSiteDomain()
	return site[0]
#end - getBlackListSite

def getCurrentURL(request = None):
	request = request or GlobalRequestMiddleware.getRequest()
	if request:
		referrer = request.get_full_path()
		if not referrer.startswith("http"):
			referrer = "http://{}{}".format(getSiteDomain(request), referrer)
		return referrer
	#end if
	
	return ""
#end - getCurrentURL

# 生成白名单跳转链接
def getGateway(url, referrer = "", request = None):
	if not referrer:
		request = request or GlobalRequestMiddleware.getRequest()
		if request:
			referrer = request.get_full_path()
			if not referrer.startswith("http"):
				referrer = "http://{}{}".format(getSiteDomain(request), referrer)
	#end if
	
	return "http://{}/gateway?url={}&referrer={}".format(getBlackListSite(), urllib.parse.quote(url), urllib.parse.quote(referrer))
#end - getGateway

class DatabaseAppsRouter(object):
	def db_for_read(self, model, **hints):
		connection = settings.DATABASE_MAPPING.get(getSiteDomain())
		logger.debug("DatabaseAppsRouter.db_for_read site is {}".format(connection))
		return connection
	#end db_for_read
	
	def db_for_write(self, model, **hints):
		connection = settings.DATABASE_MAPPING.get(getSiteDomain())
		logger.debug("DatabaseAppsRouter.db_for_write site is {}".format(connection))
		return connection
	#end db_for_read
	
	def allow_relation(self, obj1, obj2, **hints):
		return None
	#end allow_relation
	
	def allow_migrate(self, db, app_label, model_name=None, **hints):
		return None
	#end allow_migrate
#end - DatabaseAppsRouter



# encoding: utf-8
"""
@project: djangoModel->service_register
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 对外开放服务调用注册白名单
@created_time: 2023/1/12 14:29
"""

import xj_invoice
from xj_invoice.services import invoice_service

# 对外服务白名单
register_list = [
    {
        # 发票流程写入
        "service_name": "invoice_add",
        "pointer": invoice_service.InvoiceService.add
    },

]


# 遍历注册
def register():
    for i in register_list:
        setattr(xj_invoice, i["service_name"], i["pointer"])


if __name__ == '__main__':
    register()



from django.db import models
from django.utils import timezone


# Create your models here.
class InvoiceType(models.Model):
    id = models.AutoField(verbose_name='ID', primary_key=True)
    invoice_type_code = models.CharField(verbose_name='发票类型状态码', max_length=128)
    invoice_type_value = models.CharField(verbose_name='发票类型状态名', max_length=128)
    description = models.CharField(verbose_name='描述', max_length=128)

    class Meta:
        db_table = 'invoice_type'
        verbose_name_plural = "发票-发票类型表"

    def __str__(self):
        return f"{self.id}"


class InvoiceStatus(models.Model):
    id = models.AutoField(verbose_name='ID', primary_key=True)
    invoice_status = models.CharField(verbose_name='发票状态码', max_length=128)
    invoice_status_value = models.CharField(verbose_name='发票状态名', max_length=128)
    description = models.CharField(verbose_name='描述', max_length=128)

    class Meta:
        db_table = 'invoice_status'
        verbose_name_plural = "发票-发票状态表"

    def __str__(self):
        return f"{self.id}"


class InvoiceExternalStatus(models.Model):
    id = models.AutoField(verbose_name='ID', primary_key=True)
    external_status_code = models.CharField(verbose_name='外经证状态码', max_length=128)
    external_status_value = models.CharField(verbose_name='外经证状态名', max_length=128)
    description = models.CharField(verbose_name='描述', max_length=128)

    class Meta:
        db_table = 'invoice_external_status'
        verbose_name_plural = "发票-外经证状态表"

    def __str__(self):
        return f"{self.id}"


class InvoiceToEnroll(models.Model):
    id = models.AutoField(verbose_name='ID', primary_key=True)
    invoice_id = models.IntegerField(verbose_name='发票id', primary_key=False, help_text='')
    enroll_id = models.IntegerField(verbose_name='报名id', primary_key=False, help_text='')

    class Meta:
        db_table = 'invoice_to_enroll'
        verbose_name_plural = "发票-映射"

    def __str__(self):
        return f"{self.id}"


class InvoiceExternal(models.Model):
    id = models.AutoField(verbose_name='ID', primary_key=True)
    thread_id = models.IntegerField(verbose_name='合同ID（信息id）', primary_key=False, help_text='')
    external_number = models.CharField(verbose_name='外经证编号', max_length=128, unique=True, blank=True, null=True,
                                       db_index=True,
                                       help_text='')
    external_price = models.DecimalField(verbose_name='开具金额', max_digits=32, decimal_places=2, blank=True,
                                         null=True)
    create_date = models.DateTimeField(verbose_name='开具日期', auto_now_add=True)
    tax_date = models.DateTimeField(verbose_name='税单日期', auto_now_add=True)
    end_date = models.DateTimeField(verbose_name='结束日期', auto_now_add=True)
    external_address = models.CharField(verbose_name='外经证地址', max_length=128, blank=True, null=True, help_text='')
    remark = models.CharField(verbose_name='备注', max_length=128, blank=True, null=True, help_text='')
    external_status = models.ForeignKey(InvoiceExternalStatus, verbose_name='外经证状态', on_delete=models.DO_NOTHING,
                                        help_text='')
    return_date = models.DateTimeField(verbose_name='退回日期', auto_now_add=True)
    destroy_name = models.CharField(verbose_name='缴销作废状态', max_length=128, blank=True, null=True, help_text='')
    destroy_date = models.DateTimeField(verbose_name='缴销作废日期', auto_now_add=True)

    class Meta:
        db_table = 'invoice_external'
        verbose_name_plural = "发票-外经证表"

    def __str__(self):
        return f"{self.id}"


class InvoiceHeader(models.Model):
    id = models.AutoField(verbose_name='ID', primary_key=True)
    user_id = models.IntegerField(verbose_name='所属用户ID', primary_key=False, help_text='')
    head_type = models.CharField(verbose_name='抬头类型 PERSONS个人 CORPORATIONS企业', max_length=128, blank=True,
                                 null=False,
                                 help_text='')
    company_name = models.CharField(verbose_name='公司名称', max_length=128, blank=True, null=True,
                                    help_text='')
    tax_number = models.CharField(verbose_name='发票税号', max_length=128, unique=True, blank=True, null=True,
                                  db_index=True,
                                  help_text='')
    address = models.CharField(verbose_name='地址', max_length=128, blank=True, null=True, help_text='')
    account_opening_bank = models.CharField(verbose_name='开户行', max_length=128, blank=True, null=True, help_text='')
    account = models.CharField(verbose_name='账户', max_length=128, blank=True, null=True, help_text='')
    phone_number = models.CharField(verbose_name='手机号', max_length=128, blank=True, null=True, help_text='')
    is_delete = models.BooleanField(verbose_name='是否删除', default=False, blank=True, null=True, )
    sort = models.IntegerField(verbose_name='排序', primary_key=False, help_text='', default=0, )

    class Meta:
        db_table = 'invoice_header'
        verbose_name_plural = "发票-抬头"

    def __str__(self):
        return f"{self.id}"


class Invoice(models.Model):
    """ 8、*invoice_invoice 发票表 """
    id = models.AutoField(verbose_name='平台ID', primary_key=True, help_text='必填。自动生成。')
    user_id = models.IntegerField(verbose_name='开票用户', primary_key=False, help_text='')
    thread_id = models.IntegerField(verbose_name='合同ID（信息id）', primary_key=False, help_text='')
    enroll_id_list = models.CharField(verbose_name='报名列表', max_length=128, blank=True, null=True,
                                      db_index=True,
                                      help_text='要开票的报名表数据')
    invoice_time = models.DateTimeField(verbose_name='开票时间', null=True)
    invoice_type = models.ForeignKey(InvoiceType, verbose_name='发票类型', on_delete=models.DO_NOTHING, help_text='')
    invoice_status = models.ForeignKey(InvoiceStatus, verbose_name='发票状态', on_delete=models.DO_NOTHING,
                                       help_text='', default=1)
    invoice_header = models.ForeignKey(InvoiceHeader, verbose_name='发票抬头', on_delete=models.DO_NOTHING,
                                       help_text='')

    invoice_number = models.CharField(verbose_name='发票号码', max_length=128, unique=True, blank=True, null=True,
                                      db_index=True,
                                      help_text='')
    invoice_price = models.DecimalField(verbose_name='开票金额', max_digits=32, decimal_places=2, blank=True, null=True)
    tax_rate = models.CharField(verbose_name='发票税率', max_length=128, blank=True, null=True, help_text='')
    invoice_tax = models.DecimalField(verbose_name='税额', max_digits=32, decimal_places=2, blank=True, null=True)
    invoice_untaxed = models.DecimalField(verbose_name='不含税金额', max_digits=32, decimal_places=2, blank=True,
                                          null=True)

    tax_number = models.CharField(verbose_name='税务登记号', max_length=128, blank=True, db_index=True, null=True,
                                  help_text='')

    operator_user_id = models.IntegerField(verbose_name='开票操作员', primary_key=False, help_text='')
    invoicing_party = models.CharField(verbose_name='开票方', max_length=128, blank=True, db_index=True, null=True,
                                       help_text='')
    remark = models.CharField(verbose_name='备注', max_length=128, blank=True, null=True, help_text='')
    destroy_name = models.CharField(verbose_name='退回作废状态', max_length=128, blank=True, null=True, help_text='')
    destroy_date = models.DateTimeField(verbose_name='退回作废日期', default=timezone.now, help_text='')
    destroy_reason = models.CharField(verbose_name='退回作废原因', max_length=128, blank=True, null=True, help_text='')
    destroy_operator_id = models.IntegerField(verbose_name='退回操作员', primary_key=False, help_text='')

    invoice_voucher = models.CharField(verbose_name='开票凭证', max_length=128, blank=True, null=True, help_text='')
    receive_email = models.EmailField(verbose_name='接收邮箱', unique=True, blank=True, help_text='')
    receive_phone = models.CharField(verbose_name='接收手机号', max_length=128, blank=True, null=True, help_text='')
    invoice_snapshot = models.JSONField(verbose_name='发票快照', blank=True, null=True, help_text='')
    create_time = models.DateTimeField(verbose_name='创建时间', auto_now_add=True)
    update_time = models.DateTimeField(verbose_name='修改时间', auto_now_add=True)

    class Meta:
        db_table = 'invoice_invoice'
        verbose_name_plural = "01. 发票 - 发票表"

    def __str__(self):
        return f"{self.id}"


class InvoiceExtendData(models.Model):
    """ 5、invoice_extend_data扩展字段数据表 """

    class Meta:
        db_table = 'invoice_extend_data'
        verbose_name_plural = '发票- 扩展字段数据表'

    invoice_id = models.OneToOneField(verbose_name='发票ID', to=Invoice, related_name="invoice_extend_data",
                                      db_column='invoice_id',
                                      primary_key=True, on_delete=models.DO_NOTHING, help_text='')
    field_1 = models.CharField(verbose_name='自定义字段_1', max_length=255, blank=True, null=True, help_text='')
    field_2 = models.CharField(verbose_name='自定义字段_2', max_length=255, blank=True, null=True, help_text='')
    field_3 = models.CharField(verbose_name='自定义字段_3', max_length=255, blank=True, null=True, help_text='')
    field_4 = models.CharField(verbose_name='自定义字段_4', max_length=255, blank=True, null=True, help_text='')
    field_5 = models.CharField(verbose_name='自定义字段_5', max_length=255, blank=True, null=True, help_text='')
    field_6 = models.CharField(verbose_name='自定义字段_6', max_length=255, blank=True, null=True, help_text='')
    field_7 = models.CharField(verbose_name='自定义字段_7', max_length=255, blank=True, null=True, help_text='')
    field_8 = models.CharField(verbose_name='自定义字段_8', max_length=255, blank=True, null=True, help_text='')
    field_9 = models.CharField(verbose_name='自定义字段_9', max_length=255, blank=True, null=True, help_text='')
    field_10 = models.CharField(verbose_name='自定义字段_10', max_length=255, blank=True, null=True, help_text='')
    field_11 = models.CharField(verbose_name='自定义字段_11', max_length=255, blank=True, null=True, help_text='')
    field_12 = models.CharField(verbose_name='自定义字段_12', max_length=255, blank=True, null=True, help_text='')
    field_13 = models.CharField(verbose_name='自定义字段_13', max_length=255, blank=True, null=True, help_text='')
    field_14 = models.CharField(verbose_name='自定义字段_14', max_length=255, blank=True, null=True, help_text='')
    field_15 = models.CharField(verbose_name='自定义字段_15', max_length=255, blank=True, null=True, help_text='')
    field_16 = models.CharField(verbose_name='自定义字段_16', max_length=255, blank=True, null=True, help_text='')
    field_17 = models.CharField(verbose_name='自定义字段_17', max_length=255, blank=True, null=True, help_text='')
    field_18 = models.CharField(verbose_name='自定义字段_18', max_length=255, blank=True, null=True, help_text='')
    field_19 = models.CharField(verbose_name='自定义字段_19', max_length=255, blank=True, null=True, help_text='')
    field_20 = models.CharField(verbose_name='自定义字段_20', max_length=255, blank=True, null=True, help_text='')
    field_21 = models.CharField(verbose_name='自定义字段_21', max_length=255, blank=True, null=True, help_text='')
    field_22 = models.CharField(verbose_name='自定义字段_22', max_length=255, blank=True, null=True, help_text='')
    field_23 = models.CharField(verbose_name='自定义字段_23', max_length=255, blank=True, null=True, help_text='')
    field_24 = models.CharField(verbose_name='自定义字段_24', max_length=255, blank=True, null=True, help_text='')
    field_25 = models.CharField(verbose_name='自定义字段_25', max_length=255, blank=True, null=True, help_text='')
    field_26 = models.CharField(verbose_name='自定义字段_26', max_length=255, blank=True, null=True, help_text='')
    field_27 = models.CharField(verbose_name='自定义字段_27', max_length=255, blank=True, null=True, help_text='')
    field_28 = models.CharField(verbose_name='自定义字段_28', max_length=255, blank=True, null=True, help_text='')
    field_29 = models.CharField(verbose_name='自定义字段_29', max_length=255, blank=True, null=True, help_text='')
    field_30 = models.CharField(verbose_name='自定义字段_30', max_length=255, blank=True, null=True, help_text='')

    def __str__(self):
        return f"{self.invoice_id}"

    def short_field_1(self):
        if self.field_1 and len(self.field_1) > 25:
            return f"{self.field_1[0:25]}..."
        return self.field_1

    short_field_1.short_description = '自定义字段1'

    def short_field_2(self):
        if self.field_2 and len(self.field_2) > 25:
            return f"{self.field_2[0:25]}..."
        return self.field_2

    short_field_2.short_description = '自定义字段2'

    def short_field_3(self):
        if self.field_3 and len(self.field_3) > 25:
            return f"{self.field_3[0:25]}..."
        return self.field_3

    short_field_3.short_description = '自定义字段3'

    def short_field_4(self):
        if self.field_4 and len(self.field_4) > 25:
            return f"{self.field_4[0:25]}..."
        return self.field_4

    short_field_4.short_description = '自定义字段4'

    def short_field_5(self):
        if self.field_5 and len(self.field_5) > 25:
            return f"{self.field_5[0:25]}..."
        return self.field_5

    short_field_5.short_description = '自定义字段5'

    def short_field_6(self):
        if self.field_6 and len(self.field_6) > 25:
            return f"{self.field_6[0:25]}..."
        return self.field_6

    short_field_6.short_description = '自定义字段6'

    def short_field_7(self):
        if self.field_7 and len(self.field_7) > 25:
            return f"{self.field_7[0:25]}..."
        return self.field_7

    short_field_7.short_description = '自定义字段7'

    def short_field_8(self):
        if self.field_8 and len(self.field_8) > 25:
            return f"{self.field_8[0:25]}..."
        return self.field_8

    short_field_8.short_description = '自定义字段8'

    def short_field_9(self):
        if self.field_9 and len(self.field_9) > 25:
            return f"{self.field_9[0:25]}..."
        return self.field_9

    short_field_9.short_description = '自定义字段9'

    def short_field_10(self):
        if self.field_10 and len(self.field_10) > 25:
            return f"{self.field_10[0:25]}..."
        return self.field_10

    short_field_10.short_description = '自定义字段10'

    def short_field_11(self):
        if self.field_11 and len(self.field_11) > 25:
            return f"{self.field_11[0:25]}..."
        return self.field_11

    short_field_11.short_description = '自定义字段11'

    def short_field_12(self):
        if self.field_12 and len(self.field_12) > 25:
            return f"{self.field_12[0:25]}..."
        return self.field_12

    short_field_12.short_description = '自定义字段12'

    def short_field_13(self):
        if self.field_13 and len(self.field_13) > 25:
            return f"{self.field_13[0:25]}..."
        return self.field_13

    short_field_13.short_description = '自定义字段13'

    def short_field_14(self):
        if self.field_14 and len(self.field_14) > 25:
            return f"{self.field_14[0:25]}..."
        return self.field_14

    short_field_14.short_description = '自定义字段14'

    def short_field_15(self):
        if self.field_15 and len(self.field_15) > 25:
            return f"{self.field_15[0:25]}..."
        return self.field_15

    short_field_15.short_description = '自定义字段15'

    def short_field_16(self):
        if self.field_16 and len(self.field_16) > 25:
            return f"{self.field_16[0:25]}..."
        return self.field_16

    short_field_16.short_description = '自定义字段16'

    def short_field_17(self):
        if self.field_17 and len(self.field_17) > 25:
            return f"{self.field_17[0:25]}..."
        return self.field_17

    short_field_17.short_description = '自定义字段17'

    def short_field_18(self):
        if self.field_18 and len(self.field_18) > 25:
            return f"{self.field_18[0:25]}..."
        return self.field_18

    short_field_18.short_description = '自定义字段18'

    def short_field_19(self):
        if self.field_19 and len(self.field_19) > 25:
            return f"{self.field_19[0:25]}..."
        return self.field_19

    short_field_19.short_description = '自定义字段19'

    def short_field_20(self):
        if self.field_20 and len(self.field_20) > 25:
            return f"{self.field_20[0:25]}..."
        return self.field_20

    short_field_20.short_description = '自定义字段20'

    def short_field_21(self):
        if self.field_21 and len(self.field_21) > 25:
            return f"{self.field_21[0:25]}..."
        return self.field_21

    short_field_21.short_description = '自定义字段21'

    def short_field_22(self):
        if self.field_22 and len(self.field_22) > 25:
            return f"{self.field_22[0:25]}..."
        return self.field_22

    short_field_22.short_description = '自定义字段22'

    def short_field_23(self):
        if self.field_23 and len(self.field_23) > 25:
            return f"{self.field_23[0:25]}..."
        return self.field_23

    short_field_23.short_description = '自定义字段23'

    def short_field_24(self):
        if self.field_24 and len(self.field_24) > 25:
            return f"{self.field_24[0:25]}..."
        return self.field_24

    short_field_24.short_description = '自定义字段24'

    def short_field_25(self):
        if self.field_25 and len(self.field_25) > 25:
            return f"{self.field_25[0:25]}..."
        return self.field_25

    short_field_25.short_description = '自定义字段25'

    def short_field_26(self):
        if self.field_26 and len(self.field_26) > 25:
            return f"{self.field_26[0:25]}..."
        return self.field_26

    short_field_26.short_description = '自定义字段26'

    def short_field_27(self):
        if self.field_27 and len(self.field_27) > 25:
            return f"{self.field_27[0:25]}..."
        return self.field_27

    short_field_27.short_description = '自定义字段27'

    def short_field_28(self):
        if self.field_28 and len(self.field_28) > 25:
            return f"{self.field_28[0:25]}..."
        return self.field_28

    short_field_28.short_description = '自定义字段28'

    def short_field_29(self):
        if self.field_29 and len(self.field_29) > 25:
            return f"{self.field_29[0:25]}..."
        return self.field_29

    short_field_29.short_description = '自定义字段29'

    def short_field_30(self):
        if self.field_30 and len(self.field_30) > 25:
            return f"{self.field_30[0:25]}..."
        return self.field_30

    short_field_30.short_description = '自定义字段30'


# 扩展字段表。用于声明扩展字段数据表中的(有序)字段具体对应的什么键名。注意：扩展字段是对分类的扩展，而不是主类别的扩展
class InvoiceExtendField(models.Model):
    """  6、InvoiceExtendField 扩展字段表 """

    class Meta:
        db_table = 'invoice_extend_field'
        verbose_name_plural = '发票 - 扩展字段表'
        # ordering = ['-invoice_type_id']

    field_index_choices = [
        ("field_1", "field_1"),
        ("field_2", "field_2"),
        ("field_3", "field_3"),
        ("field_4", "field_4"),
        ("field_5", "field_5"),
        ("field_6", "field_6"),
        ("field_7", "field_7"),
        ("field_8", "field_8"),
        ("field_9", "field_9"),
        ("field_10", "field_10"),
        ("field_11", "field_11"),
        ("field_12", "field_12"),
        ("field_13", "field_13"),
        ("field_14", "field_14"),
        ("field_15", "field_15"),
        ("field_16", "field_16"),
        ("field_17", "field_17"),
        ("field_18", "field_18"),
        ("field_19", "field_19"),
        ("field_20", "field_20"),
        ("field_21", "field_21"),
        ("field_22", "field_22"),
        ("field_23", "field_23"),
        ("field_24", "field_24"),
        ("field_25", "field_25"),
        ("field_26", "field_26"),
        ("field_27", "field_27"),
        ("field_28", "field_28"),
        ("field_29", "field_29"),
        ("field_30", "field_30")
    ]
    type_choices = [
        ("string", "string"),
        ("int", "int"),
        ("float", "float"),
        ("bool", "bool"),
        ("select", "select"),
        ("radio", "radio"),
        ("checkbox", "checkbox"),
        ("date", "date",),
        ("time", "time",),
        ("datetime", "datetime"),
        ("moon", "moon"),
        ("year", "year"),
        ("color", "color"),
        ("file", "file"),
        ("image", "image"),
        ("switch", "switch"),
        ("cascader", "cascader"),
        ("plain", "plain"),
        ("textarea", "textarea"),
        ("text", "text"),
        ("number", "number"),
        ("upload", "upload"),
        ("password", "password"),
    ]

    id = models.AutoField(verbose_name='ID', primary_key=True, help_text='')
    # 数据库生成classify_id字段
    # invoice_type = models.ForeignKey(verbose_name='发票ID', null=True, blank=True, to=InvoiceType,
    #                                  db_column='invoice_type_id', related_name='+', on_delete=models.DO_NOTHING,
    #                                  help_text='')
    field = models.CharField(verbose_name='自定义字段', max_length=255, help_text='')  # 眏射ThreadExtendData表的键名
    field_index = models.CharField(verbose_name='冗余字段', max_length=255, help_text='',
                                   choices=field_index_choices)  # 眏射ThreadExtendData表的键名
    value = models.CharField(verbose_name='字段介绍', max_length=255, null=True, blank=True, help_text='')
    type = models.CharField(verbose_name='字段类型', max_length=255, blank=True, null=True, choices=type_choices,
                            help_text='')
    unit = models.CharField(verbose_name='参数单位', max_length=255, blank=True, null=True, help_text='')
    config = models.JSONField(verbose_name='字段配置', blank=True, null=True, default=dict, help_text='')
    description = models.CharField(verbose_name='数据配置', max_length=255, blank=True, null=True, default=dict,
                                   help_text='')
    default = models.CharField(verbose_name='默认值', max_length=2048, blank=True, null=True, help_text='')

    def __str__(self):
        return f"{self.id}"



import json
from decimal import Decimal

from django.db.models import F
from django.forms import model_to_dict
from ..models import *
from utils.custom_tool import filter_result_field
from ..utils.utility_method import convert_dict_to_json, parse_integers, get_current_time, replace_dict_key, \
    search_in_dict_list, extract_values
from xj_invoice.utils.join_list import JoinList
from xj_thread.services.thread_list_service import ThreadListService
from xj_user.services.user_service import UserService
from ..services.invoice_extend_service import InvoiceExtendService
from ..utils.custom_tool import format_params_handle, force_transform_type, dynamic_load_class, \
    write_to_log


# 自定义序列化函数
def decimal_default(obj):
    if isinstance(obj, Decimal):
        return str(obj)  # 将 Decimal 对象转换为字符串
    raise TypeError(f"Object of type {obj.__class__.__name__} is not JSON serializable")


class InvoiceService:

    @staticmethod
    def batch_add(params):
        params, is_void = force_transform_type(variable=params, var_type="dict", default={})
        invoice_list = params.get("invoice_list")
        try:
            for item in invoice_list:
                invoice, err = InvoiceService.add(item)
                if err:
                    return None, err
        except Exception as e:
            return None, f'''{str(e)} in "{str(e.__traceback__.tb_frame.f_globals["__file__"])}" : Line {str(
                e.__traceback__.tb_lineno)}'''

        return None, None

    @staticmethod
    def add(params: dict = None, **kwargs):
        """
        发票添加
        :param params: 添加参数子字典
        :param kwargs:
        :return:
        """
        # 参数整合与空值验证
        params, is_void = force_transform_type(variable=params, var_type="dict", default={})
        kwargs, is_void = force_transform_type(variable=kwargs, var_type="dict", default={})
        params.update(kwargs)
        invoice_type = InvoiceType.objects.filter(invoice_type_code=params['invoice_type_code']).first()
        if not invoice_type:
            return None, "发票类型不存在"
        invoice_status_code = params.get("invoice_status_code", "APPLY")
        invoice_status = InvoiceStatus.objects.filter(invoice_status=invoice_status_code).first()
        if not invoice_status:
            return None, "发票状态不存在"

        params['invoice_type_id'] = invoice_type.id
        params['invoice_status_id'] = invoice_status.id
        # 过滤主表修改字段
        try:
            main_form_data = format_params_handle(
                param_dict=params.copy(),
                is_remove_empty=True,
                filter_filed_list=[
                    "user_id|int",
                    "thread_id|int",
                    "enroll_id_list",
                    "invoice_time|date",
                    "invoice_type_id|int",
                    "invoice_price|float",
                    "invoice_number",
                    "tax_rate",
                    "invoice_tax",
                    "invoice_untaxed",
                    "tax_number",
                    "operator_user_id|int",
                    'remark',
                    "invoice_status_id",
                    "destroy_name",
                    "destroy_date",
                    "destroy_reason",
                    "destroy_operator_id",
                    "receive_email",
                    "receive_phone",
                    "invoice_snapshot|dict",
                    "invoicing_party",
                    "invoice_voucher"

                ],
                alias_dict={},
                is_validate_type=True
            )
        except ValueError as e:
            # 模型字段验证
            return None, str(e)

        # 必填参数校验
        must_keys = ["thread_id", "user_id"]
        for i in must_keys:
            if not params.get(i, None):
                return None, str(i) + " 必填"
        invoice_snapshot = {}
        # 处理快照信息
        company_address = params.get("company_address", "")  # 公司地址
        company_phone = params.get("company_phone", "")  # 公司电话
        deposit_bank = params.get("deposit_bank", "")  # 开户银行
        deposit_account = params.get("deposit_account", "")  # 开户账户
        enroll_id_list = []
        invoice_snapshot['account_info'] = {
            "company_address": company_address,
            "company_phone": str(company_phone),
            "deposit_bank": deposit_bank,
            "deposit_account": str(deposit_account)
        }
        if main_form_data.get("enroll_id_list", ""):
            enroll_id_list = parse_integers(main_form_data.get("enroll_id_list", ""))
            # 载入模块
            EnrollServices, import_err = dynamic_load_class(import_path="xj_enroll.service.enroll_services",
                                                            class_name="EnrollServices")
            assert not import_err
            enroll, err = EnrollServices.enroll_list(params={"id_list": enroll_id_list})
            if enroll:
                thread_id_list = [item.get("thread_id", None) for item in enroll['list']]
                thread_list, err = ThreadListService.search(thread_id_list,
                                                            filter_fields=['title', 'classify_value', 'classify_name'])
                if thread_list:
                    enroll['list'] = JoinList(enroll['list'], thread_list, "thread_id", "id").join()
                invoice_snapshot['enroll'] = enroll['list']

        # if main_form_data.get("user_id", ""):
        #     user_id_list = [main_form_data.get("user_id", "")]
        #     user_list, err = UserService.user_list(allow_user_list=user_id_list)
        #     if user_list:
        #         invoice_snapshot['user'] = user_list['list'][0]

        if main_form_data.get("thread_id", ""):
            thread_id_list = [main_form_data.get("thread_id", "")]
            thread_list, err = ThreadListService.search(thread_id_list)
            if thread_list:
                invoice_snapshot['thread'] = thread_list[0]
        # main_form_data['invoice_snapshot'] = json.loads(
        #     json.dumps(invoice_snapshot, default=decimal_default, ensure_ascii=False))
        main_form_data['invoice_snapshot'] = json.loads(convert_dict_to_json(invoice_snapshot))
        # IO操作
        try:
            # 主表插入数据
            # main_form_data['create_time'] = InvoiceService.get_current_time()
            invoice = Invoice.objects.create(**main_form_data)
            # 扩展表 插入或更新
            add_extend_res, err = InvoiceExtendService.create_or_update(params, invoice.id)
            if err:
                write_to_log(prefix="发票--扩展表 插入或更新失败", err_obj=err)

            for enroll_id in enroll_id_list:
                InvoiceToEnroll.objects.create(**{"invoice_id": invoice.id, "enroll_id": enroll_id})

            # 载入模块
            EnrollServices, import_err = dynamic_load_class(import_path="xj_enroll.service.enroll_services",
                                                            class_name="EnrollServices")
            assert not import_err
            enroll, enroll_err = EnrollServices.enroll_edit(params={"finance_invoicing_code": "INVOICING"},
                                                            search_param={"enroll_id_list": enroll_id_list
                                                                          })
            if enroll_err:
                write_to_log(
                    prefix="开票功能",
                    content="报名id:" + str(main_form_data.get("enroll_id_list", "")),
                    err_obj=enroll_err
                )
        except Exception as e:
            return None, f'''{str(e)} in "{str(e.__traceback__.tb_frame.f_globals["__file__"])}" : Line {str(
                e.__traceback__.tb_lineno)}'''

        return {"id": invoice.id}, None

    @staticmethod
    def edit(params: dict = None, invoice_id=None, search_param: dict = None, **kwargs):
        """
        发票编辑
        :param params: 修改的参数
        :param invoice_id: 需要修改的发票主键
        :param search_param: 搜索参数, 服务层根据信息等其他搜索字段检索到需要修改的数据
        :return: data, err
        """
        # 空值检查
        params, is_pass = force_transform_type(variable=params, var_type="dict", default={})
        invoice_id, is_pass = force_transform_type(variable=invoice_id, var_type="int")
        search_param, is_pass = force_transform_type(variable=search_param, var_type="dict", default={})
        if not invoice_id and not search_param:
            return None, "无法找到要修改数据，请检查参数"

        # 搜索字段过滤
        if search_param:
            search_param = format_params_handle(
                param_dict=search_param,
                filter_filed_list=[
                    "invoice_id|int", "invoice_id_list|list", "thread_id|int", "thread_id_list|list",
                ],
                alias_dict={"invoice_id_list": "id__in", "thread_id_list": "thread_id__in"}
            )
        # 修改内容检查处理
        try:
            main_form_data = format_params_handle(
                param_dict=params,
                is_validate_type=True,
                is_remove_empty=True,
                filter_filed_list=[
                    "user_id|int",
                    "thread_id|int",
                    "enroll_id_list",
                    "invoice_time|date",
                    "invoice_type_id|int",
                    "invoice_price|float",
                    "invoice_number",
                    "tax_rate",
                    "invoice_tax",
                    "invoice_untaxed",
                    "tax_number",
                    "operator_user_id|int",
                    'remark',
                    "invoice_status_id",
                    "destroy_name",
                    "destroy_date",
                    "destroy_reason",
                    "destroy_operator_id",
                    "receive_email",
                    "receive_phone",
                    "invoice_snapshot|dict",
                    "invoicing_party",
                    "invoice_voucher"

                ],
            )
        except ValueError as e:
            return None, str(e)
        # 构建ORM，检查是否存在可修改项目
        invoice_obj = Invoice.objects
        if invoice_id:
            invoice_obj = invoice_obj.filter(id=invoice_id)
        elif search_param:
            invoice_obj = invoice_obj.filter(**search_param)

        update_total = invoice_obj.count()
        if update_total == 0:
            return None, "没有找到可修改项目"

        # IO 操作
        try:
            main_form_data.pop("invoice_id", None)
            main_form_data['update_time'] = get_current_time()
            invoice_obj.update(**main_form_data)
            add_extend_res, err = InvoiceExtendService.create_or_update(params, invoice_id)
            if err:
                write_to_log(prefix="发票--扩展表 插入或更新失败", err_obj=err)
        except Exception as e:
            return None, "修改异常:" + str(e)
        return invoice_obj.first().to_json(), None

    @staticmethod
    def list(params):
        not_matching_count = 0
        page = int(params['page']) - 1 if 'page' in params else 0
        size = int(params['size']) if 'size' in params else 10
        invoice = Invoice.objects
        # invoice = invoice.order_by('-invoice_time')
        invoice = invoice.order_by('-id')
        if params.get("is_all", 0) >= 1 or params.get("user_id_list", ""):
            params.pop("user_id")
        if params.get("enroll_id_list", []):
            id_list_set = InvoiceToEnroll.objects.filter(enroll_id__in=params.get("enroll_id_list")).values(
                "invoice_id")
            id_list = extract_values(id_list_set, "invoice_id")
            invoice = invoice.filter(id__in=id_list)
        params = format_params_handle(
            param_dict=params,
            is_remove_empty=True,
            filter_filed_list=[
                "id|int", "id_list|list", "thread_id|int", "user_id|int", "invoice_type_code",
                "thread_id_list|list", "user_id_list|list","head_type",
                "invoice_number", "create_time_start", "create_time_end",
                "invoice_time_start", "invoice_time_end", "invoice_status_code"
            ],
            split_list=["thread_id_list", "id_list", "user_id_list"],
            alias_dict={
                "create_time_start": "create_time_gte", "create_time_end": "create_time__lte",
                "invoice_time_start": "invoice_time__gte", "invoice_time_end": "invoice_time__lte",
                "thread_id_list": "thread_id__in", "id_list": "id__in", "user_id_list": "user_id__in",
                "invoice_type_code": "invoice_type__invoice_type_code__iexact",
                "invoice_status_code": "invoice_status__invoice_status__iexact",
                "head_type": "invoice_header__head_type__iexact"
            },
        )
        invoice = invoice.extra(select={'invoice_time': 'DATE_FORMAT(invoice_time, "%%Y-%%m-%%d %%H:%%i:%%s")'})
        invoice = invoice.annotate(invoice_type_code=F("invoice_type__invoice_type_code"),
                                   invoice_type_value=F("invoice_type__invoice_type_value"))
        invoice = invoice.annotate(invoice_status_code=F("invoice_status__invoice_status"),
                                   invoice_status_value=F("invoice_status__invoice_status_value"))
        invoice = invoice.annotate(head_type=F("invoice_header__head_type"),
                                   company_name=F("invoice_header__company_name"),
                                   head_tax_number=F("invoice_header__tax_number"),
                                   address=F("invoice_header__address"),
                                   phone_number=F("invoice_header__phone_number"),
                                   account_opening_bank=F("invoice_header__account_opening_bank"),
                                   account=F("invoice_header__account"),)
        invoice = invoice.filter(**params).values()
        total = invoice.count()
        #
        current_page_set = invoice[page * size: page * size + size] if page >= 0 and size > 0 else invoice
        res_list = []
        for i, it in enumerate(current_page_set):
            it['order'] = page * size + i + 1
            it['invoice_time'] = it['invoice_time'].strftime("%Y-%m-%d %H:%M:%S") if it['invoice_time'] else it[
                'invoice_time']
            it['create_time'] = it['create_time'].strftime("%Y-%m-%d %H:%M:%S") if it['create_time'] else it[
                'create_time']
            it['update_time'] = it['update_time'].strftime("%Y-%m-%d %H:%M:%S") if it['update_time'] else it[
                'update_time']

            res_list.append(it)

        data = res_list

        # ========== 四、相关前置业务逻辑处理 ==========

        # ========== 五、翻页 ==========

        user_id_list = [item.get("user_id", "") for item in res_list]
        user_list, err = UserService.user_list(allow_user_list=user_id_list)
        if user_list:
            data = JoinList(res_list, user_list['list'], "user_id", "user_id").join()

        data = filter_result_field(
            result_list=filter_result_field(  # 扩展字段替换
                result_list=data,
                alias_dict={"id": "invoice_id", "invoice_status_value": "invoice_status"},
            ),
        )
        # ======================= 信息模块适配 start ==============================
        thread_id_list = [item.get("thread_id", None) for item in data]
        thread_list, err = ThreadListService.search(thread_id_list, filter_fields="title")
        if thread_list:
            data = JoinList(data, thread_list, "thread_id", "id").join()
        # ======================= 信息模块适配 end ==============================

        # ======================= 发票扩展字段适配 start ==============================
        extend_id_list = [item.get("invoice_id", None) for item in data]
        extend_list, err = InvoiceExtendService.get_extend_info(extend_id_list)
        if extend_list:
            data = JoinList(data, extend_list, "invoice_id", "invoice_id_id").join()
        # ======================= 发票扩展字段适配 end ==============================
        return {'size': int(size), 'page': int(page + 1), 'total': total, 'list': data, }, None

    @staticmethod
    def detail(params):
        invoice_id = params.get("invoice_id")
        invoice_number = params.get("invoice_number")
        if not invoice_id and not invoice_number:
            return None, "发票信息不能为空"
        if invoice_id:
            invoice = Invoice.objects.filter(id=invoice_id)
        else:
            invoice = Invoice.objects.filter(invoice_number=invoice_number)

        invoice = invoice.extra(select={'invoice_time': 'DATE_FORMAT(invoice_time, "%%Y-%%m-%%d %%H:%%i:%%s")',
                                        'create_time': 'DATE_FORMAT(create_time, "%%Y-%%m-%%d %%H:%%i:%%s")',
                                        'update_time': 'DATE_FORMAT(update_time, "%%Y-%%m-%%d %%H:%%i:%%s")'})

        invoice = invoice.first()
        if not invoice:
            return None, "无法找到要查询的数据，请检查参数"
        data = model_to_dict(invoice)
        data = replace_dict_key(data, 'id', 'invoice_id')
        invoice_type = InvoiceType.objects.filter(id=data['invoice_type']).first()
        if invoice_type:
            data.update(model_to_dict(invoice_type))
            data = replace_dict_key(data, 'id', 'invoice_type_id')
        invoice_status = InvoiceStatus.objects.filter(id=data['invoice_status']).first()
        if invoice_status:
            data.update(model_to_dict(invoice_status))
            data = replace_dict_key(data, 'id', 'invoice_status_id')
        user_id_list = [data.get("user_id", None)]
        user_list, err = UserService.user_list(allow_user_list=user_id_list)
        if user_list:
            data.update(user_list['list'][0])
        thread_id_list = [data.get("thread_id", None)]
        thread_list, err = ThreadListService.search(thread_id_list, filter_fields="title")
        if thread_list:
            data.update(thread_list[0])
        extend_id_list = [data.get("invoice_id", None)]
        extend_list, err = InvoiceExtendService.get_extend_info(extend_id_list)
        if extend_list:
            data.update(extend_list[0])
        return data, None

    @staticmethod
    def examine_approve(params):
        invoice_id = params.get("invoice_id", 0)
        invoice_status = params.get("invoice_status_code", "")
        invoice_number = params.get("invoice_number", "")
        invoice_time = params.get("invoice_time", "")
        data = {}
        if not invoice_id:
            return None, "发票id不能为空"
        if invoice_status not in ["APPLY", "INVOIC", "RETURN", "CANCEL"]:
            return None, "未支持发票状态"

        invoice_status = InvoiceStatus.objects.filter(invoice_status=invoice_status).first()
        if not invoice_status:
            return None, "发票状态不存在"
        data['invoice_status_id'] = invoice_status.id
        data['invoice_number'] = invoice_number
        data['invoice_time'] = invoice_time
        invoice_set = Invoice.objects.filter(id=invoice_id)
        invoice_update = invoice_set.update(**data)
        if not invoice_update:
            return None, "审核失败"
        invoice_obj = invoice_set.first()
        invoice_obj = model_to_dict(invoice_obj)
        enroll_id_list = parse_integers(invoice_obj.get("enroll_id_list", ""))
        # 载入模块
        EnrollServices, import_err = dynamic_load_class(import_path="xj_enroll.service.enroll_services",
                                                        class_name="EnrollServices")
        assert not import_err
        enroll, err = EnrollServices.enroll_edit(params={"finance_invoicing_code": "INVOICED"},
                                                 search_param={"enroll_id_list": enroll_id_list
                                                               })
        if err:
            write_to_log(
                prefix="开票功能",
                content="报名id:" + str(invoice_obj.get("enroll_id_list", "")),
                err_obj=err
            )
        return invoice_obj, None



# _*_coding:utf-8_*_

import os, logging, time, json, copy

from django.views.decorators.http import require_http_methods
from rest_framework.decorators import api_view
from rest_framework.views import APIView

from ..services.invoice_header_service import InvoiceHeaderService
from ..services.invoice_type_service import InvoiceTypeService
from ..utils.custom_tool import request_params_wrapper, flow_service_wrapper
from ..utils.user_wrapper import user_authentication_wrapper
from ..utils.model_handle import util_response
from ..services.invoice_service import InvoiceService

logger = logging.getLogger(__name__)


class InvoiceHeaderApi(APIView):  # 或继承(APIView)

    # 发票抬头添加
    @require_http_methods(['POST'])
    @user_authentication_wrapper
    @request_params_wrapper
    def add(self, *args, user_info, request_params, **kwargs, ):
        params = request_params
        pay_mode_set, err = InvoiceHeaderService.add(params)
        if err is None:
            return util_response(data=pay_mode_set)
        return util_response(err=47767, msg=err)

    #  发票抬头修改
    @require_http_methods(['PUT'])
    @user_authentication_wrapper
    @request_params_wrapper
    def edit(self, *args, user_info, request_params, **kwargs, ):
        params = request_params
        pay_mode_set, err = InvoiceHeaderService.edit(params)
        if err is None:
            return util_response(data=pay_mode_set)
        return util_response(err=47767, msg=err)

    #  发票抬头列表
    @require_http_methods(['GET'])
    @user_authentication_wrapper
    @request_params_wrapper
    def list(self, *args, user_info, request_params, **kwargs, ):
        params = request_params
        # ============   字段验证处理 start ============
        pay_mode_set, err = InvoiceHeaderService.list(params)
        if err is None:
            return util_response(data=pay_mode_set)
        return util_response(err=47767, msg=err)

    #  发票抬头列表
    @require_http_methods(['GET'])
    @user_authentication_wrapper
    @request_params_wrapper
    def detail(self, *args, user_info, request_params, **kwargs, ):
        params = request_params
        # ============   字段验证处理 start ============
        pay_mode_set, err = InvoiceHeaderService.detail(params)
        if err is None:
            return util_response(data=pay_mode_set)
        return util_response(err=47767, msg=err)

    # 发票抬头删除
    @require_http_methods(['DELETE'])
    @user_authentication_wrapper
    @request_params_wrapper
    def delete(self, *args, user_info, request_params, **kwargs, ):
        params = request_params
        pay_mode_set, err = InvoiceHeaderService.delete(params)
        if err is None:
            return util_response(data=pay_mode_set)
        return util_response(err=47767, msg=err)
